import unittest

import os
import sys
import pwd
import dbus
import dbus.decorators
import dbus.glib
import time
import gobject

loop = gobject.MainLoop()
bus = dbus.SessionBus()

run_debugger = False

class WorkraveTestBase(unittest.TestCase):
        
    def get_num_autostart_workraves(self):
        return 3
    
    def start_workrave(self, instance, clean = True):

        name = "workrave" + str(instance)
        tmpdir = "/tmp/" + name + "/"
        pidfile = tmpdir + "pid"
        cwd = os.getcwd();

        env = os.environ
        env["WORKRAVE_TEST"]="1"
        env["WORKRAVE_FAKE"]="1"
        env["WORKRAVE_DBUS_NAME"] = "org.workrave.Workrave" + str(instance)
        env["WORKRAVE_HOME"] = tmpdir

        if clean:
            try:
                os.system("rm -rf " + tmpdir + ".workrave");
                os.mkdir(tmpdir)
            except:
                pass

        newpid = os.fork()
        if newpid == 0:
            out_log = file(name + ".out", 'w+')
            dev_null = file('/dev/null', 'r')

            os.dup2(out_log.fileno(), sys.stdout.fileno())
            os.dup2(out_log.fileno(), sys.stderr.fileno())
            os.dup2(dev_null.fileno(), sys.stdin.fileno())
            
            os.execlpe("/sbin/start-stop-daemon",
                       "/sbin/start-stop-daemon",               
                       "--quiet",
                       "--start",
                       "--pidfile", pidfile,
                       "--make-pidfile",
                       "--exec",
                       #"/usr/bin/valgrind",
                       #"--", "-v", "--trace-children=yes", "--leak-check=full",
                       #"--show-reachable=yes",
                       #"--log-file=" + cwd +"/val." + str(instance),  
                       cwd + "/ui/app/text/src/workrave",
                       env)
                  

    def stop_workrave(self, instance):
        name = "workrave" + str(instance)
        tmpdir = "/tmp/" + name + "/"
        pidfile = tmpdir + "pid"

        newpid = os.fork()
        if newpid == 0:
            os.execlp("/sbin/start-stop-daemon",
                      "/sbin/start-stop-daemon",
                      "--stop",
                      "--pidfile", pidfile)

    def launch(self, num, clean = True):
        for i in range(num):
            self.start_workrave(i + 1, clean)
        time.sleep(2)

        self.wr = []
        self.wrd = []
        self.core = []
        self.network = []
        self.config = []
        self.debug = []

        for i in range(num):
            self.wr.append(bus.get_object("org.workrave.Workrave" + str(i + 1), "/org/workrave/Workrave/Core"))
            self.wrd.append(bus.get_object("org.workrave.Workrave" + str(i + 1), "/org/workrave/Workrave/Debug"))

            self.core.append   (dbus.Interface(self.wr[i], "org.workrave.CoreInterface"))
            self.network.append(dbus.Interface(self.wr[i], "org.workrave.NetworkInterface"))
            self.config.append (dbus.Interface(self.wr[i], "org.workrave.ConfigInterface"))
            self.debug.append  (dbus.Interface(self.wrd[i], "org.workrave.DebugInterface"))

        if run_debugger:
            for i in range(1, num + 1):
                os.system("cat /tmp/workrave" + str(i) + "/pid")
            print "Attach debugger and press return."
            sys.stdin.readline();
            time.sleep(1)

        time.sleep(1)
        for i in range(num):
            # Start listening for incoming connections.
            self.network[i].Listen(2701 + i)

            self.config[i].SetBool("timers/micro_pause/enabled", True)
            self.config[i].SetInt("timers/micro_pause/auto_reset", 20)
            self.config[i].SetInt("timers/micro_pause/limit", 60)
            self.config[i].SetInt("timers/micro_pause/snooze", 180)
            self.config[i].SetBool("timers/rest_break/enabled", False)
            self.config[i].SetBool("timers/daily_limit/enabled", False)

        self.num_running = num;

    def kill(self):
        time.sleep(2)
        if run_debugger:
            print "Press return"
            sys.stdin.readline();

        for i in range(1, self.num_running + 1):
            self.stop_workrave(i)
        
    def setUp(self):

        self.num_running = 0
        self.wr = []
        self.wrd = []
        self.core = []
        self.network = []
        self.config = []
        self.debug = []
        self.connected = False
        
        num = self.get_num_autostart_workraves()
        self.launch(num);
        time.sleep(1)

    def tearDown(self): 
        self.kill()
        time.sleep(4)



#!/usr/bin/python
#
# Copyright (C) 2007, 2008 Rob Caelers <robc@krandor.nl>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#
"""
Workrave Activity Monitor based on face detection.

Based on OpenCV examples.

Original C example implementation by:  ?
Python example implementation by: Roman Stanchak
"""

import os
import sys
import pwd
import dbus
import dbus.decorators
import dbus.glib
import time
import gobject

import ConfigParser
from optparse import OptionParser

from opencv.cv import *
from opencv.highgui import *

class FaceActivityMonitor:

    def __init__(self):
        self.init_variables()
        self.init_options()
        self.init_dbus()
        self.init_face_detect()
        
    def cleanup(self):
        if self.show:
            cvDestroyWindow("result")

    def init_variables(self):
        self.cascade_name     = "/usr/share/opencv/haarcascades/haarcascade_frontalface_default.xml"
        self.device           = 0
        self.show             = False
        self.verbose          = False
        self.face_threshold   = 2 
        self.noface_threshold = 5

        # Parameters for haar detection
        # From the API:
        # The default parameters (scale_factor=1.1, min_neighbors=3, flags=0) are tuned 
        # for accurate yet slow object detection. For a faster operation on real video 
        # images the settings are: 
        # scale_factor=1.2, min_neighbors=2, flags=CV_HAAR_DO_CANNY_PRUNING, 
        # min_size=<minimum possible face size
        self.min_size         = cvSize(20,20)
        self.image_scale      = 1.2
        self.haar_scale       = 1.2
        self.min_neighbors    = 2
        self.haar_flags       = CV_HAAR_DO_CANNY_PRUNING

        self.workrave = None
        self.cascase  = None
        self.capture  = None
        self.storage = None
        self.last_time = 0;
        self.frame_copy = None
        self.count_face = 0
        self.count_noface = 0
        self.ignore = False
        
    def init_options(self):
        usage = "usage: %prog [options]"    
        parser = OptionParser(usage=usage)
        parser.add_option("-d", "--devicenum",
                          dest="device", default=0,
                          help="/dev/video device number, default = 0",
                          )

        parser.add_option("-c", "--cascade",
                          dest="cascade_name", default=None,
                          help="Haar cascade to use",
                          )

        parser.add_option("-s", "--show",
                          action="store_true", dest="show", default=True,
                          help="Show detection",
                          )

        parser.add_option("-v", "--verbose",
                          action="store_true", dest="verbose", default=False,
                          help="Verbose message",
                          )
        (options, args) = parser.parse_args()

        self.device = options.device
        self.show = options.show
        self.verbose = options.verbose
        
        if options.cascade_name:
            self.cascade_name = options.cascade_name


    def init_dbus(self):
        bus = dbus.SessionBus()
        obj = bus.get_object("org.workrave.Workrave", "/org/workrave/Workrave/Core")

        self.workrave = dbus.Interface(obj, "org.workrave.CoreInterface")

        self.workrave.connect_to_signal("MicrobreakChanged",
                                        self.microbreak_signal, sender_keyword='sender')
        self.workrave.connect_to_signal("RestbreakChanged",
                                        self.restbreak_signal, sender_keyword='sender')
        self.workrave.connect_to_signal("DailylimitChanged",
                                        self.dailylimit_signal, sender_keyword='sender')
        
    def init_face_detect(self):
        # the OpenCV API says this function is obsolete, but we can't
        # cast the output of cvLoad to a HaarClassifierCascade, so use this anyways
        # the size parameter is ignored
        self.cascade = cvLoadHaarClassifierCascade(self.cascade_name, cvSize(1,1))
    
        if not self.cascade:
            print "ERROR: Could not load classifier cascade"
            sys.exit(-1)
            
        self.capture = cvCreateCameraCapture(int(self.device))
        self.storage = cvCreateMemStorage(0)

        if self.show:
            cvNamedWindow("result", 1)
        

    def detect_and_draw(self, image):
        gray = cvCreateImage(cvSize(image.width,image.height), 8, 1)
        small_image = cvCreateImage(cvSize(cvRound(image.width/self.image_scale),
                                           cvRound(image.height/self.image_scale)),
                                    8, 1)
    
        cvCvtColor(image, gray, CV_BGR2GRAY)
        cvResize(gray, small_image, CV_INTER_LINEAR)
        cvEqualizeHist(small_image, small_image)
     
        cvClearMemStorage(self.storage)

        faces = cvHaarDetectObjects(small_image,
                                    self.cascade,
                                    self.storage,
                                    self.haar_scale,
                                    self.min_neighbors,
                                    self.haar_flags,
                                    self.min_size)
        if self.show and faces:
            for r in faces:
                pt1 = cvPoint(int(r.x*self.image_scale), int(r.y*self.image_scale))
                pt2 = cvPoint(int((r.x+r.width)*self.image_scale), int((r.y+r.height)*self.image_scale))
                cvRectangle(image, pt1, pt2, CV_RGB(255,0,0), 3, 8, 0)

            cvShowImage("result", image)

        return faces and faces.total > 0

    def iteration(self):
        frame = cvQueryFrame(self.capture)
        if not frame:
            return False
             
        if frame.origin == IPL_ORIGIN_TL:
            user_detected = self.detect_and_draw(frame)
        else:
            if not self.frame_copy:
                self.frame_copy = cvCreateImage(cvSize(frame.width,frame.height),
                                                IPL_DEPTH_8U, frame.nChannels)
                
            cvFlip(frame, self.frame_copy, 0)
            user_detected = self.detect_and_draw(self.frame_copy)

        self.inform_workrave(user_detected)

        cvWaitKey(5)
        

    def inform_workrave(self, user_detected):

        if self.ignore:
            return
        
        if user_detected:
            self.count_face += 1
            self.count_noface = 0
        else:
            self.count_noface += 1
            self.count_face = 0
        
        if self.count_face >= self.face_threshold:

            if self.verbose and self.count_face == self.face_threshold:
                print "Reporting user presence"
                
            now = time.time()
            if now > self.last_time + 5:
                self.workrave.ReportActivity("facedetect", True)
                self.last_time = now

        if self.count_noface == self.noface_threshold:
            if self.verbose:
                print "Reporting user absence"
            self.workrave.ReportActivity("facedetect", False)

    def microbreak_signal(self, progress, sender=None):
        self.break_signal("microbreak", progress)
        
    def restbreak_signal(self, progress, sender=None):
        self.break_signal("restbreak", progress)

    def dailylimit_signal(self, progress, sender=None):
        self.break_signal("dailylimit", progress)

    def break_signal(self, breakid, progress, sender=None):
        if progress != "none":
            self.count_face = 0
            self.count_noface = 0
            self.ignore = True;
            self.workrave.ReportActivity("facedetect", False)
        else:
            self.ignore = False;
        
        if self.verbose:
            if progress == "prelude":
                print "Ignoring user presence: %s warning" % breakid
            elif progress == "break":
                print "Ignoring user presence: %s started" % breakid
            elif progress == "none":
                print "Resuming presence monitoring: Break %s idle" % breakid
            else:
                print "Unknown progress for %s: %s" % (breakid, progress)


class Timeout(gobject.Timeout):
    def __init__(self, context, interval, func, *args):
        gobject.Timeout.__init__(self, interval, priority=gobject.PRIORITY_HIGH)
        self.set_callback(self.callback, func, *args)
        self.attach(context)

    def callback(self, func, *args):
        func(*args)
        return True

if __name__ == '__main__':

    am = FaceActivityMonitor()

    context = gobject.MainContext()
    loop = gobject.MainLoop(context=context)

    source = Timeout(context, 1000, am.iteration)

    loop.run()



#!/usr/bin/env python3

polling_interval = 1.0
date_format = "%a %b %d, %I:%M %p"

import json
import sys
import time
from datetime import datetime

from workrave_break_info import fmt_timer_plain, WorkraveBreakInfo

wr_break_info = WorkraveBreakInfo(color_dict = {
    "default": ("#000000", "#87CEEB"),
    "close to break": ("#000000", "#FFA500"),
    "overdue": ("#FFFFFF", "#FF0000")
})

print('{ "version": 1, "click_events": true}\n[')

while True:
    print("[")

    for timer_type in ("microbreak", "restbreak", "dailylimit"):
        timer_info = wr_break_info.get_timer_info(timer_type)

        if timer_info.enabled:
            print(f'   {{"full_text": "{fmt_timer_plain(*timer_info)}", '
                  f'"name": "workrave", "instance": "wr_{timer_type}", '
                  f'"color": "{timer_info.colors[0]}", '
                  f'"background": "{timer_info.colors[1]}"}},') # Note comma at end

    print(f'   {{"full_text": "{datetime.now().strftime(date_format)}"}}')

    print("],", flush = True) 

    time.sleep(polling_interval)



#!/usr/bin/env python3

from dasbus.connection import SessionMessageBus

session_bus = SessionMessageBus()

wr_ui = session_bus.get_proxy('org.workrave.Workrave',
                              '/org/workrave/Workrave/UI',
                              interface_name = "org.workrave.AppletInterface")

wr_ui.MenuAction('workrave.open')



#!/usr/bin/python
import os
import sys
import pwd
import dbus
import dbus.decorators
import dbus.glib
import time
import gobject

# Prints out all signals workrave sends regarding changes in break states
class WorkraveDBus:

    def __init__(self):
        bus = dbus.SessionBus()
        obj = bus.get_object("org.workrave.Workrave", "/org/workrave/Workrave/Core")

        self.workrave = dbus.Interface(obj, "org.workrave.CoreInterface")
        self.config = dbus.Interface(obj, "org.workrave.ConfigInterface")

        self.workrave.connect_to_signal("MicrobreakChanged",
                                   self.on_microbreak_changed, sender_keyword='sender')
        self.workrave.connect_to_signal("RestbreakChanged",
                                   self.on_restbreak_changed, sender_keyword='sender')
        self.workrave.connect_to_signal("DailylimitChanged",
                                   self.on_dailylimit_signal, sender_keyword='sender')

    def on_microbreak_changed(self, progress, sender=None):
        self.on_break_changed("microbreak", progress)
        
    def on_restbreak_changed(self, progress, sender=None):
        self.on_break_changed("restbreak", progress)

    def on_dailylimit_signal(self, progress, sender=None):
        self.on_break_changed("dailylimit", progress)

    def on_break_changed(self, breakid, progress, sender=None):
        if progress == "prelude":
            print "Break warning %s" % breakid
        elif progress == "break":
            print "Break %s started" % breakid
        elif progress == "none":
            print "Break %s idle" % breakid
            self.on_break_idle(breakid)
        else:
            print "Unknown progress for %s: %s" % (breakid, progress)

    def on_break_idle(self, breakid):
        if breakid == "microbreak":
            configid = "micro_pause"
        elif breakid == "restbreak":
            configid = "rest_break"
        elif breakid == "dailylimit":
            configid = "daily_limit"
            
        limit = self.config.GetInt("/timers/%s/limit" % configid)[0]
        autoreset = self.config.GetInt("timers/%s/auto_reset" % configid)[0]

        if self.workrave.GetTimerIdle(breakid) >= autoreset:
            print "Break %s taken" % breakid
        elif self.workrave.GetTimerElapsed(breakid) < limit:
            print "Break %s skipped" % breakid
        else:
            print "Break %s postponed"

if __name__ == '__main__':
    d = WorkraveDBus()

    loop = gobject.MainLoop()
    loop.run()



#!/usr/bin/env python3

from dasbus.connection import SessionMessageBus
from collections.abc import Sequence
from collections import namedtuple

#########################################################################
#
# For those who want to edit this script's behavior (especially to add
# a new format) or to use it as a module ...
#
# The "fmt_timer_{$FORMAT}" function, where $FORMAT is a format type
# (plain, waybar, etc.), formats the string for an individual timer,
# i.e.  microbreak, rest break, or daily limit. This function takes
# the following inputs to describe the timer:
#
#  * enabled: a Boolean indicating if the timer is enabled
#
#  * time_left: in seconds, the time left until the next break, or for
#    the case of the daily limit, the time left for computer activity
#    for the day
#
#  * limit: in seconds, the time between breaks, or for the case of
#    the daily limit, the daily limit itself
#
#  * timer_type: a string with one of the following values:
#    "microbreak", "restbreak", or "dailylimit".
#
#  * timer_state: a string with one of the following values:
#    "default", "close to break", and "overdue".
#
#  * colors: a tuple of two strings, the first with a string
#    indicating the foreground (or text) color, and the second
#    indicating the background color. The colors indicate the value of
#    timer_state.
#
# The "fmt_all_timers_{$FORMAT}" function takes a list of strings,
# each of which has information on an individual timer, and produces a
# string that contains informations about all of the timers
# described in that list.
#
# Here's how these functions are used. For example, suppose that:
#
#   * The interval between microbreaks is 5 minutes and the time to
#     the next microbreak is 3 minutes.
#
#   * The interval between rest breaks is 50 minutes and the time to
#     the next rest break is 15 minutes.
#
#   * The daily limit is 4 hours, and time left for computer activity
#     is 2.5 hours.
#
# Then fmt_timer_plain will generate the string "M 3:00/5:00" for the
# microbreak, "R 15:00/50:00" for the rest break, and
# "D 4:00:00/2:30:00" for the daily limit. fmt_all_timers_plain will
# receive the list of strings ["M 3:00/5:00", "R 15:00/50:00",
# "D 4:00:00/2:30:00"] and output the following string:
#
# "M 3:00/5:00 R 15:00/50:00 D 4:00:00/2:30:00"
#
# The dictionary fmt_info_dict associates a string that labels the
# format with a tuple containing the "fmt_timer_{$FORMAT}"
# function and the "fmt_all_timers_{$FORMAT}" function.
# The first key in fmt_info_dict is the default format (i.e.,
# "plain").
#
# To add, say, format "foo", define the functions fmt_timer_foo and
# fmt_all_timers_foo, and then add the following entry to fmt_info_dict:
#
#   foo: (fmt_timer_foo, fmt_all_timers_foo)
#
# This script can also be used as a module, and in addition to the
# aforementioned functions, it also has the class WorkraveBreakInfo,
# whose constructor has the following signature:
#
# class WorkraveBreakInfo(color_dict = None,
#                         get_timer_state_func = None)
#
# The optional keyword argument color_dict is a dictionary with three
# keys, which correspond to the aforementioned three states of the
# timer: "default", "close to break", and "overdue", and the values
# corresponding to each of those keys is a two-element sequence whose
# first element indicates a foreground color and whose second element
# indicates a background color. Both elements are strings containing
# some color specification suitable for the chosen fmt_timer_func.
#
# The optional keyword argument get_timer_state_func is a function
# that returns one of the strings "default", "close to break", or
# "overdue", depending on its two arguments: "time_left" and
# "limit". The first is the time left until a break starts, and the
# second is the minimum time between breaks (i.e., a *limit* on the
# amount of time that one can be active before the break starts). By
# default, a timer is considered "close to break" if time_left/limit
# <= 0.1 and "overdue" if time_left is negative.
#
# The class WorkraveBreakInfo has the following public methods:
#
#     * get_timer_info(): Takes one argument, the break type, which
#       can be "microbreak", "restbreak", or "dailylimit". It returns
#       a named tuple of type TimerInfo whose fields are all the
#       arguments needed for a "fmt_timer_{$FORMAT}" function:
#       "enabled", "time_left", "limit", "timer_type", "timer_state",
#       and "colors". Give such a tuple named "foo" and a
#       "fmt_timer_{$FORMAT}" function named "fmt_timer_baz", one can
#       use this tuple with that function simply by unpacking it,
#       i.e., doing this: fmt_timer_baz(*foo).
#
#     * open_workrave(): This just opens a Workrave status window if
#       one is not already open. It basically does the same thing as
#       workrave-open.py.
#
# This class does *not* do any polling. That is either the job of the
# module when it is used as a script, or the job of whatever Python
# script wishes to use the class -- if it needs to do any polling at
# all.
#
# There is another free function in this module,
# seconds_to_time_str(), which takes a time interval in seconds as an
# argument, and returns a string in the format "%M:%S" (or "%H:%M:%S"
# for time intervals longer than a hour). It can be used in a custom
# "fmt_timer_{$FORMAT}" function.
#
#########################################################################

def fmt_timer_plain(enabled, time_left, limit, timer_type, timer_state, colors):

    if not enabled: return ""

    time_left_str = seconds_to_time_str(time_left)
    limit_str = seconds_to_time_str(limit)

    prefix = {"microbreak": "M",
              "restbreak": "R",
              "dailylimit": "D"}[timer_type]
    
    return f"{prefix}: {time_left_str}/{limit_str}"

def fmt_all_timers_plain(timer_strs):
    return " ".join(timer_str for timer_str in timer_strs if timer_str)

def fmt_timer_waybar(enabled, time_left, limit, timer_type, timer_state, colors):

    if not enabled: return ""
    
    timer_str = fmt_timer_plain(enabled, time_left, limit, timer_type, timer_state, colors)
    
    return f"<span color='{colors[0]}' bgcolor='{colors[1]}'>{timer_str}</span>"

def fmt_all_timers_waybar(timer_strs):
    all_timers_str = " ".join(timer_str for timer_str in timer_strs if timer_str)
    
    return  f"""{{"text": "{all_timers_str}", "tooltip": "Workrave"}}"""

def fmt_timer_yambar(enabled, time_left, limit, timer_type, timer_state, colors):

    timer_str = fmt_timer_plain(True, time_left, limit, timer_type, timer_state, colors)
    
    return "\n".join((f"{timer_type}|string|{timer_str.replace('|', '/')}",
                      f"{timer_type}_enabled|bool|{str(enabled).lower()}",
                      f'{timer_type}_state|string|{timer_state}'))

def fmt_all_timers_yambar(timer_strs):
    all_timers_str = "\n".join(timer_strs)
    return f"{all_timers_str}\n"

def fmt_timer_json(enabled, time_left, limit, timer_type, timer_state, colors):
    limit_str_key_prefix = (f"daily" if timer_type == "dailylimit"
                            else f"{timer_type}_")
    
    vars_str = ", ".join([f'"{timer_type}_enabled": {str(enabled).lower()}',
                          f'"{timer_type}_left_in_seconds": {time_left}',
                          f'"{limit_str_key_prefix}limit_in_seconds": {limit}',
                          f'"{timer_type}_left_str": "{seconds_to_time_str(time_left)}"',
                          f'"{limit_str_key_prefix}limit_str": "{seconds_to_time_str(limit)}"',
                          f'"{timer_type}_state": "{timer_state}"',
                          f'"{timer_type}_fgcol": "{colors[0]}"',
                          f'"{timer_type}_bgcol": "{colors[1]}"'])

    return f"{{{vars_str}}}"

def fmt_all_timers_json(timer_strs):
    all_timers_str = ", ".join(timer_str[1:-1] # [1:-1] removes
                                               # initial and final
                                               # braces.
                               for timer_str in timer_strs)
    
    return f"{{{all_timers_str}}}"

fmt_info_dict = {
    "plain": (fmt_timer_plain, fmt_all_timers_plain),
    "waybar": (fmt_timer_waybar, fmt_all_timers_waybar),
    "yambar": (fmt_timer_yambar, fmt_all_timers_yambar),
    "json": (fmt_timer_json, fmt_all_timers_json)
}

#### Shouldn't need to edit anything below here. ####

def seconds_to_time_str(tot_num_sec):
    abs_tot_num_sec = abs(tot_num_sec)
    sign = "" if tot_num_sec >= 0 else "-"

    if abs_tot_num_sec >= 3600:
        num_hr, rem_num_sec =  divmod(abs_tot_num_sec, 3600)
        num_min, num_sec = divmod(rem_num_sec, 60)

        return f"{sign}{num_hr}:{num_min:02d}:{num_sec:02d}"
    else:
        num_min, num_sec = divmod(abs_tot_num_sec, 60)
        return f"{sign}{num_min}:{num_sec:02d}"

_default_color_dict = {
    "default": ["black", "skyblue"],
    "close to break": ["black", "orange"],
    "overdue": ["white", "red"]
}

def _default_get_timer_state(time_left, limit):
    if time_left < 0:
        return "overdue"
    elif time_left/limit <= 0.1:
        return "close to break"
    else:
        return "default"

TimerInfo = namedtuple(
    "TimerInfo",
    "enabled, time_left, limit, timer_type, timer_state, colors"
)
    
class WorkraveBreakInfo:
    def __init__(self, *,
                 color_dict = None,
                 get_timer_state_func = None):
        
        self.session_bus = SessionMessageBus()

        self._wr_core = self.session_bus.get_proxy(
            'org.workrave.Workrave',
            '/org/workrave/Workrave/Core',
            interface_name = "org.workrave.CoreInterface"
        )

        self._wr_cfg = self.session_bus.get_proxy(
            'org.workrave.Workrave',
            '/org/workrave/Workrave/Core',
            interface_name = "org.workrave.ConfigInterface"
        )

        self._timer_enabled_paths = {
            "microbreak": "/breaks/micro-pause/enabled",
            "restbreak": "/breaks/rest-break/enabled",
            "dailylimit": "/breaks/daily-limit/enabled"
        }

        self._timer_limit_paths = {
            "microbreak": "/timers/micro_pause/limit",
            "restbreak": "/timers/rest_break/limit",
            "dailylimit": "/timers/daily_limit/limit"
        }

        self.color_dict = (_default_color_dict
                           if color_dict is None
                           else color_dict)

        self.get_timer_state = (_default_get_timer_state
                                if get_timer_state_func is None
                                else get_timer_state_func)

        self._check_color_dict()

    def _check_color_dict(self):

        for key in ("default", "close to break", "overdue"):
            if key not in self.color_dict:
                raise ValueError(f"'{key}' is required key in "
                                 "'color_dict' argument")

            if not isinstance(self.color_dict[key], Sequence):
                raise ValueError(f"Value associated with key '{key}' "
                                 "must be a sequence")

            if len(self.color_dict[key]) != 2:
                raise ValueError("Length of sequence associated with "
                                 f"key '{key}' is not 2. Sequence is "
                                 f"{self.color_dict[key]}")

    def get_timer_info(self, timer_type):

        timer_enabled_path = self._timer_enabled_paths[timer_type]

        timer_limit_path = self._timer_limit_paths[timer_type]

        enabled = self._wr_cfg.GetBool(timer_enabled_path)[0]

        limit = self._wr_cfg.GetInt(timer_limit_path)[0]
        time_left = limit - self._wr_core.GetTimerElapsed(timer_type)

        timer_state = self.get_timer_state(time_left, limit)
        colors = self.color_dict[timer_state]

        return TimerInfo(enabled, time_left, limit,
                         timer_type, timer_state, colors)

    def open_workrave(self):
        wr_ui = self.session_bus.get_proxy(
            'org.workrave.Workrave',
            '/org/workrave/Workrave/UI',
            interface_name = "org.workrave.AppletInterface"
        )

        wr_ui.MenuAction('workrave.open')

def main():
    import argparse
    import time

    output_fmts = tuple(fmt_info_dict.keys())

    parser = argparse.ArgumentParser()

    parser.add_argument("-i", "--polling-interval", type = float, default = 1.0,
                        help = "Time interval for polling Workrave, in seconds "
                        "[default = %(default)s]")

    parser.add_argument("-f", "--format",
                        choices = output_fmts, default = output_fmts[0],
                        help = "Format for output [default = %(default)s]")

    parser.add_argument("--colors-default", nargs = 2,
                        default = _default_color_dict["default"],
                        help = "Default timer colors (if color is used in the output "
                        "format). First color is text color, second is background color. "
                        "[default = %(default)s]")

    parser.add_argument("--colors-close-to-break", nargs = 2,
                        default = _default_color_dict["close to break"],
                        help = "Timer color when close to break time (if color is used in "
                        "the output format).  First color is text color, second is "
                        "background color.  [default = %(default)s]")

    parser.add_argument("--colors-overdue", nargs = 2,
                        default = _default_color_dict["overdue"],
                        help = "Timer colors when interval between breaks exceeded (if "
                        "color is used in the output format. First color is text color, "
                        "second is background color. [default = %(default)s]")

    args = parser.parse_args()

    fmt_timer_func, fmt_all_timers_func = fmt_info_dict[args.format]

    wr_break_info = WorkraveBreakInfo(color_dict = {
        "default": args.colors_default,
        "close to break": args.colors_close_to_break,
        "overdue": args.colors_overdue
    })
    
    while True:
        print(
            fmt_all_timers_func(
                [fmt_timer_func(*wr_break_info.get_timer_info(timer_type))
                 for timer_type in ("microbreak", "restbreak", "dailylimit")]
            ), flush = True
        )

        time.sleep(args.polling_interval)

if __name__ == "__main__":
    main()
